#- parameters and sets ----------------------------------------------------------

param schedule:= season + "_Schedule.txt";
param table:= season + "_Table.txt";
param skipRowsMatchplan := ( matchday - 1 ) * 9;
param skipRowsTable := ( matchday - 2 ) * 18;

defnumb matchesLeft(roundrobin) := if roundrobin == "single" then 153 - skipRowsMatchplan else 306 - skipRowsMatchplan end;

set S := { read schedule as "<5s, 6s>" skip skipRowsMatchplan use matchesLeft(round_robin) comment "#"}; # remaining schedule
set M := { read table as "<2s>" skip skipRowsTable use 18 comment "#"};                                  # teams

param P[M] := read table as "<2s> 10n" skip skipRowsTable use 18 comment "#";                            # points of each team

#- information and checks -------------------------------------------------------

do print "====================================================";
do print "season: ", season;
do print "team: ", team;
do print "matchday: ", matchday;
do print "matches left: ", matchesLeft(round_robin);
do print "problem: ", problem;
do print "round_robin: ", round_robin;
do print "====================================================";
do print "Skip data rows in match plan:       ", skipRowsMatchplan;
do print "Skip data rows in list of standings: ", skipRowsTable;
do forall <i,j> in S do print "home team: ", i, " away team: ", j;
do forall <m> in M do print m, " ", P[m];

do check card(M) == 18;
do check card(S) == matchesLeft(round_robin);
do check problem == "bestCase" or problem == "worstCase";
do check round_robin == "single" or round_robin == "double";

#- variables --------------------------------------------------------------------

var x[S] binary;                                    # home victory for 1s
var y[S] binary;                                    # draw
var z[S] binary;                                    # away victory for 2s
var points[M] real >= 0 <= 3 * 2 * ( card(M) - 1 ); # number of points is bounded by zero or all wins 
var c[M] binary;                                    # counting variable
var obj real >= -18 <= 18;                          # variable representing the objective function

#- objective --------------------------------------------------------------------

maximize standing: obj;

#- constraints ------------------------------------------------------------------

subto problemObj:
# to conveniently solve the models from the command line (see README), we transformed (4b) also to a maximization problem
   if problem == "worstCase" then obj == card(M) - sum <m> in M without {<team>}: c[m] else obj == - card(M) + sum <m> in M without {<team>}: c[m] end;

subto Eqn_1: forall <i,j> in S do
# either home victory or draw or away victory of opponent
   x[i,j] + y[i,j] + z[i,j] == 1;

subto Eqn_2: forall <m> in M do
# the current points of team m plus the hypothetical points from home and away games yield the final number of points
   points[m] == P[m] + sum <m,j> in S: ( 3 * x[m,j] + y[m,j] ) + sum <i,m> in S: ( y[i,m] + 3 * z[i,m] );

subto Eqn_3a_b: forall <m> in M do
# induce correct ranking
   if problem == "worstCase" then points[team] - points[m] <= c[m] * ( P[team] - P[m] + 3 * matchesLeft(round_robin) / 9 ) else points[team] - points[m] >= ( 1 - c[m] ) * ( P[team] - P[m] - 3 * matchesLeft(round_robin) / 9 ) end;
